package com.walletkit.sdk;

import java.util.Map;

import org.apache.http.client.HttpClient;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONObject;

import com.walletkit.sdk.constants.Constants;
import com.walletkit.sdk.utils.WKUtils;

/**
 * WalletKit REST Client
 * 
 * @author WalletKit <support@walletkit.com>
 * @version 1.0
 */
public class WKRestClient implements Constants {

	private final WKAuth wkauth;
	private HttpClient httpClient;

	/**
	 * Creates a WalletKit REST client based on the credentials
	 * 
	 * @param APIKey
	 *            api key
	 * @param BrandId
	 *            brand id
	 */
	public WKRestClient(final String APIKey, final String BrandId) {
		wkauth = new WKAuth();
		wkauth.setAPIKey(APIKey);
		wkauth.setBrandId(BrandId);
		httpClient = new DefaultHttpClient();
	}

	/**
	 * Creates a pass template
	 * 
	 * @param data
	 *            the template json data
	 * 
	 * @return json string as response containing the template id
	 */
	public String createPassTemplate(final WKPassTemplate data) {
		String response = null;
		String id = null;
		final HttpPost postRequest = new HttpPost(PASS_LIST_URL);
		try {
			postRequest.addHeader(BRAND_ID, wkauth.getBrandId());
			postRequest.addHeader(API_KEY, wkauth.getAPIKey());
			final StringEntity entity = new StringEntity(data.getJson());
			entity.setContentType(CONTENT_TYPE);
			postRequest.setEntity(entity);
			final ResponseHandler<String> responseHandler = new BasicResponseHandler();
			response = httpClient.execute(postRequest, responseHandler);
			id = new JSONObject(response).getString(ID);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			postRequest.releaseConnection();
		}

		return id;
	}

	/**
	 * Creates a pass template
	 * 
	 * @param requestJson
	 *            json string containing request parameters
	 * 
	 * @return json string as response containing the template id
	 */
	public String createPassTemplate(final String requestJson) {
		String response = null;
		String id = null;
		final HttpPost postRequest = new HttpPost(PASS_LIST_URL);
		try {
			postRequest.addHeader(BRAND_ID, wkauth.getBrandId());
			postRequest.addHeader(API_KEY, wkauth.getAPIKey());
			final StringEntity entity = new StringEntity(requestJson);
			entity.setContentType(CONTENT_TYPE);
			postRequest.setEntity(entity);
			final ResponseHandler<String> responseHandler = new BasicResponseHandler();
			response = httpClient.execute(postRequest, responseHandler);
			id = new JSONObject(response).getString(ID);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			postRequest.releaseConnection();
		}

		return id;
	}

	/**
	 * Shows the list of pass templates created
	 * 
	 * @return json string as response containing the id and description of each
	 *         pass
	 */
	public String getPassTemplateList() {
		String response = null;
		final HttpGet getRequest = new HttpGet(PASS_LIST_URL);
		try {
			getRequest.addHeader(BRAND_ID, wkauth.getBrandId());
			getRequest.addHeader(API_KEY, wkauth.getAPIKey());
			final ResponseHandler<String> responseHandler = new BasicResponseHandler();
			response = httpClient.execute(getRequest, responseHandler);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			getRequest.releaseConnection();
		}
		return response;
	}

	/**
	 * Gets information about the pass template
	 * 
	 * @param templateId
	 *            the template id
	 * @return json string as response containing information about the pass
	 *         template
	 */
	public String getPassTemplateInfo(final String templateId) {
		String response = null;
		final HttpGet getRequest = new HttpGet(PASS_LIST_URL + "/" + templateId);
		try {
			getRequest.addHeader(BRAND_ID, wkauth.getBrandId());
			getRequest.addHeader(API_KEY, wkauth.getAPIKey());
			final ResponseHandler<String> responseHandler = new BasicResponseHandler();
			response = httpClient.execute(getRequest, responseHandler);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			getRequest.releaseConnection();
		}
		return response;
	}

	/**
	 * Creates a pass
	 * 
	 * @param passData
	 *            the pass details
	 * @return json String as response
	 */
	public String createPass(final WKPassData passData) {
		String response = null;
		final HttpPost postRequest = new HttpPost(PASS_LIST_URL + "/"
				+ passData.getTemplateId());
		try {
			postRequest.addHeader(BRAND_ID, wkauth.getBrandId());
			postRequest.addHeader(API_KEY, wkauth.getAPIKey());
			final StringEntity entity = new StringEntity(passData.getJson());
			entity.setContentType(CONTENT_TYPE);
			postRequest.setEntity(entity);
			final ResponseHandler<String> responseHandler = new BasicResponseHandler();
			response = httpClient.execute(postRequest, responseHandler);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			postRequest.releaseConnection();
		}
		return response;
	}

	/**
	 * Creates a pass
	 * 
	 * @param templateId
	 *            the template id
	 * @param requestJson
	 *            the request json
	 * @return json String as response containing information about pass created
	 */
	public String createPass(final String templateId, final String requestJson) {
		String response = null;
		final HttpPost postRequest = new HttpPost(PASS_LIST_URL + "/"
				+ templateId);
		try {
			postRequest.addHeader(BRAND_ID, wkauth.getBrandId());
			postRequest.addHeader(API_KEY, wkauth.getAPIKey());
			final StringEntity entity = new StringEntity(requestJson);
			entity.setContentType(CONTENT_TYPE);
			postRequest.setEntity(entity);
			final ResponseHandler<String> responseHandler = new BasicResponseHandler();
			response = httpClient.execute(postRequest, responseHandler);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			postRequest.releaseConnection();
		}
		return response;
	}

	/**
	 * Updates a pass.
	 * 
	 * @param templateId
	 *            the template id.
	 * @param filterKeys
	 *            the keys to filter data with.
	 * @param updateKeys
	 *            the keys to update.
	 * @return json string as response containing information about pass updated
	 */
	public String updatePass(final String templateId,
								final Map<String, String> filterKeys,
								final Map<String, String> updateKeys) {
		String response = null;
		String jsonStr = generateJson(templateId, filterKeys, updateKeys);
		final HttpPut putRequest = new HttpPut(PASS_LIST_URL + "/" + templateId);
		try {
			putRequest.addHeader(BRAND_ID, wkauth.getBrandId());
			putRequest.addHeader(API_KEY, wkauth.getAPIKey());
			final StringEntity entity = new StringEntity(jsonStr);
			entity.setContentType(CONTENT_TYPE);
			putRequest.setEntity(entity);
			final ResponseHandler<String> responseHandler = new BasicResponseHandler();
			response = httpClient.execute(putRequest, responseHandler);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			putRequest.releaseConnection();
		}
		return response;
	}

	/**
	 * Updates a pass
	 * 
	 * @param templateId
	 *            the template id
	 * @param requestJson
	 *            the request json
	 * @return json string as response containing information about pass updated
	 */
	public String updatePass(final String templateId, final String requestJson) {
		String response = null;
		final HttpPut putRequest = new HttpPut(PASS_LIST_URL + "/" + templateId);
		try {
			putRequest.addHeader(BRAND_ID, wkauth.getBrandId());
			putRequest.addHeader(API_KEY, wkauth.getAPIKey());
			final StringEntity entity = new StringEntity(requestJson);
			entity.setContentType(CONTENT_TYPE);
			putRequest.setEntity(entity);
			final ResponseHandler<String> responseHandler = new BasicResponseHandler();
			response = httpClient.execute(putRequest, responseHandler);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			putRequest.releaseConnection();
		}
		return response;
	}

	private String generateJson(final String templateId,
								final Map<String, String> filterKeys,
								final Map<String, String> updateKeys) {
		final JSONObject jobj = new JSONObject();
		try {
			jobj.put(ID, templateId);
			jobj.put(KEYS, WKUtils.getJson(filterKeys));
			jobj.put(UPDATE_KEYS, WKUtils.getJson(updateKeys));
		} catch (Exception e) {
			e.printStackTrace();
		}
		return jobj.toString();
	}

	/**
	 * Deletes a pass template
	 * 
	 * @param templateId
	 *            the template id
	 * @return json response indicating whether the template has been deleted or
	 *         not
	 */
	public String deletePassTemplate(final String templateId) {
		String response = null;
		final HttpDelete delRequest = new HttpDelete(PASS_LIST_URL + "/"
				+ templateId);
		try {
			delRequest.addHeader(BRAND_ID, wkauth.getBrandId());
			delRequest.addHeader(API_KEY, wkauth.getAPIKey());
			final ResponseHandler<String> responseHandler = new BasicResponseHandler();
			response = httpClient.execute(delRequest, responseHandler);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			delRequest.releaseConnection();
		}
		return response;
	}

	/**
	 * Shuts down client
	 */
	public void shutdown() {
		if (httpClient != null)
			httpClient.getConnectionManager().shutdown();
	}
}
